/*
This file is part of the Remote ImageJ.
 
(c) 2011 INSERM
This software is developed at Montpellier RIO Imaging (IFR 122), Montpellier, France (www.mri.cnrs.fr)
Developer: Volker Baecker (volker.baecker@mri.cnrs.fr) 

The Remote ImageJ allows to run ImageJ macros on a distant machine.

This software is governed by the CeCILL-B license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-B
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-B license and that you accept its terms. 
*/
package fr.cnrs.mri.remoteij.client.rmr;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;

public class RMRClientApplication extends Observable {

	public enum Aspect {
		SERVER, PORT, PING_OK, PING_FAILED, STATUS, CONNECTION_FAILURE, RESULT_CHANGED}

	private String server;
	private int port;
	private String status;
	private String macro;
	private RemoteMacroRunnerClient client;
	private Map<String, String> examples = new HashMap<String, String>();
	private String exampleMacroName;
	private RMRClientApplicationView view;
	
	public static void main(String[] args) {
		new RMRClientApplication().show();
	}
	
	public void show() {
		this.getView().setVisible(true);
	}

	public RMRClientApplicationView getView() {
		if (this.view==null) this.view = new RMRClientApplicationView(this);
		return view;
	}

	public RMRClientApplication() {
		this.setupExamples();
	}
	
	private void setupExamples() {
		examples.put("addition", "return toString(2+2);");
		examples.put("count blobs", this.getCountBlobsMacro());
		examples.put("find edges", this.getFindEdgesMacro());
		this.setExampleMacroName((String)this.examplesNames()[0]);
	}

	private String getCountBlobsMacro() {
		return "run(\"Blobs (25K)\");\n" +
				"setAutoThreshold(\"Default\");\n" +
				"run(\"Analyze Particles...\", \"size=30-Infinity circularity=0.00-1.00 show=Nothing display exclude clear\");\n" +
				"close();\n" +
				"return toString(nResults);\n";
	}

	private String getFindEdgesMacro() {
		return "fileList = call(\"fr.cnrs.mri.macro.io.IOSettings.getFileList\");\n" +  
			"files = split(fileList, \",\");\n" +
			"for (i=0; i<files.length; i++) {\n" +
		    "    open(files[i]);\n" +
		    "    run(\"Find Edges\");\n" +
		    "    outputFolder = call(\"fr.cnrs.mri.macro.io.IOSettings.getOutputFolder\");\n" +
		    "    saveAs(\"Tiff\", outputFolder + \"/\" + getTitle());\n" +  
		    "    close();\n" +
		"}\n" + 
		"return \"done\";\n";
	}
	
	public String getServer() {
		return server;
	}

	public void setServer(String text) {
		this.server = text;
		this.changed(Aspect.SERVER);
	}

	public void setPort(String aPort) {
		int port = this.port;
		try {
			port = Integer.parseInt(aPort);
			this.port = port;
		}  catch (NumberFormatException e) {
			port = this.port;
		}
		this.changed(Aspect.PORT);
	}

	public int getPort() {
		return this.port;
	}
	
	void changed(Aspect anAspect) {
		this.setChanged();
		this.notifyObservers(anAspect);
		this.clearChanged();
	}

	public void ping() {
		new Thread(new PingTask(this)).start();
	}

	public void setStatus(String status) {
		this.status = status;
		this.changed(Aspect.STATUS);
	}

	public String getStatus() {
		return this.status;
	}
	
	public RemoteMacroRunnerClient getRMRClient() {
		if (client==null) client = new RemoteMacroRunnerClient();	
		client.setServer(this.getServer());
		client.setServerPort(this.getPort());
		client.setPort(this.getPort()+1);
		return client;
	}

	public void runMacro(String macro) {
		this.macro = macro;
		new Thread(new RunMacro(this)).start();
	}

	public String getMacro() {
		return macro;
	}
	
	public String getResult() {
		return this.getRMRClient().getLastResult();
	}
	
	public Object[] examplesNames() {
		List<String> list = new ArrayList<String>((this.examples.keySet()));
		Object[] result = list.toArray();
		Arrays.sort(result);
		return result;
	}
	
	public void setExampleMacroName(String name) {
		this.exampleMacroName = name;
	}
	
	public String getExampleMacroName() {
		return exampleMacroName;
	}
	
	public String getExampleMacro() {
		return this.examples.get(exampleMacroName);
	}
}
