/*
This file is part of the Remote ImageJ.
 
(c) 2011 INSERM
This software is developed at Montpellier RIO Imaging (IFR 122), Montpellier, France (www.mri.cnrs.fr)
Developer: Volker Baecker (volker.baecker@mri.cnrs.fr) 

The Remote ImageJ allows to run ImageJ macros on a distant machine.

This software is governed by the CeCILL-B license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-B
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-B license and that you accept its terms. 
*/
package fr.cnrs.mri.remoteij.server.rmr;

import ij.IJ;
import ij.IJ.ExceptionHandler;
import java.io.CharArrayWriter;
import java.io.PrintWriter;
import java.util.Observable;

import fr.cnrs.mri.dialog.ModalDialogKiller;
import fr.cnrs.mri.macro.io.IOSettings;
import fr.cnrs.mri.remoteij.client.rmr.MacroWithSettings;
import fr.cnrs.mri.server.Server;
import fr.cnrs.mri.sijame.connection.SIJAMEClient;
import fr.cnrs.mri.sijame.connection.SIJAMEServer;
import fr.cnrs.mri.sijame.messages.AnswerMessage;
import fr.cnrs.mri.sijame.messages.MessageQueue;
import fr.cnrs.mri.sijame.messages.RequestMessage;
import fr.cnrs.mri.util.logging.LoggingUtil;

/**
 * The remote-macro-runner-server listens for incoming requests containing macros. 
 * A macro will be executed using ImageJ and the return value of the macro will
 * be send back to the remote-macro-runner-client.  
 * 
 * @author Volker Baecker
 *
 */
public class RemoteMacroRunnerServer extends Observable implements Server, ExceptionHandler {
	public enum Aspect {EXECUTED_MACRO}; 
	/**
	 * The sijame-server used for the communication.
	 */
	private SIJAMEServer sijameServer;
	private ModalDialogKiller modalDialogKiller;
	
	/**
	 * Create a new server that will listen on the given port.
	 *  
	 * @param port	the port on which the server will listen.
	 */
	public RemoteMacroRunnerServer(int port) {
		this.sijameServer = SIJAMEServer.currentForPort(port);
		this.sijameServer.addObserver(this);
	}

	public static void main(String[] args) {
		if (args.length != 1) {
			System.err.println("Usage: java RemoteMacroRunnerServer.class port_number]");
			System.exit(1);
		}
		int port = Integer.parseInt(args[0]);
		RemoteMacroRunnerServer server = new RemoteMacroRunnerServer(port);
		server.start();
	}
	
	/**
	 * Start the server and start observing the request-queue.
	 */
	public void start() {
		if (this.isRunning()) return;
		this.getModalDialogKiller().start();
		this.sijameServer.start();
		while(!sijameServer.isRunning()) {
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				LoggingUtil.getLoggerFor(this).warning(LoggingUtil.getMessageAndStackTrace(e));
			}
		}
		this.sijameServer.getRequestQueue().addObserver(this);
	}
	
	/**
	 * Stop the server.
	 */
	public void stop() {
		this.getModalDialogKiller().stop();
		this.sijameServer.shutdown();
		while(sijameServer.isRunning()) {
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				LoggingUtil.getLoggerFor(this).warning(LoggingUtil.getMessageAndStackTrace(e));
			}
		}
	}

	/**
	 * Look for incoming requests.
	 */
	@Override
	public void update(Observable sender, Object aspect) {
		if (sender==this.sijameServer && aspect.equals(SIJAMEServer.Aspect.IS_RUNNING)) {
			this.handleIsRunningChanged();
			return;
		}
		if (sender != sijameServer.getRequestQueue()) return;
		if (aspect.equals(MessageQueue.Aspect.CONTENT)) return;
		RequestMessage request = (RequestMessage) aspect;
		this.handleRequest(request);
	}

	private void handleIsRunningChanged() {
		this.changed(Server.Aspect.IS_RUNNING);
	}

	private void changed(Aspect anAspect) {
		this.setChanged();
		this.notifyObservers(anAspect);
		this.clearChanged();
	}

	private void changed(Server.Aspect anAspect) {
		this.setChanged();
		this.notifyObservers(anAspect);
		this.clearChanged();
	}
	
	public boolean isRunning() {
		return sijameServer.isRunning();
	}

	public int getNumberOfMacrosInQueue() {
		if (sijameServer==null) return 0;
		return sijameServer.getRequestQueue().size();
	}
	/**
	 * Handle the request to execute a macro.
	 * 
	 * @param request	A message requesting the execution of the contained macro.
	 */
	private void handleRequest(RequestMessage request) {
		if (request.getTopic().equals("executeMacro")) {
			MacroWithSettings macroWithSettings = (MacroWithSettings)request.getData();
			IOSettings.setInstance(macroWithSettings.getSettings());
			String result = this.executeMacro(macroWithSettings.getMacro());
			AnswerMessage answer = new AnswerMessage(result);
			SIJAMEClient client = new SIJAMEClient(request.getSenderIPAddress(), request.getSenderPort());
			client.send(answer);
			request.consume();
		}
	}

	/**
	 * Execute the given macro and answer the return value of the macro.
	 * 
	 * @param macro	The macro that will be executed
	 * @return	The return value of the macro.
	 */
	private String executeMacro(String macro) {
		IJ.setExceptionHandler(this);
		String result = IJ.runMacro(macro);
		if (result==null) result = "null";
		IJ.setExceptionHandler(null);
		this.changed(Aspect.EXECUTED_MACRO);
		return result;
	}

	@Override
	public void handle(Throwable e) {
		CharArrayWriter caw = new CharArrayWriter();
		PrintWriter pw = new PrintWriter(caw);
		e.printStackTrace(pw);
		String s = caw.toString();
		LoggingUtil.getLoggerFor(this).info(s);
	}
	
	private ModalDialogKiller getModalDialogKiller() {
		if (this.modalDialogKiller==null) this.modalDialogKiller = new ModalDialogKiller();
		return modalDialogKiller;
	}
}
